@@ -3,9 +3,10 @@ import unittest
 from arangodb.api import Client, Database, Collection, Document
 from arangodb.orm.fields import CharField, ForeignKeyField
 from arangodb.orm.models import CollectionModel
-from arangodb.query.advanced import Query
-from query.simple import SimpleQuery
-from transaction.controller import Transaction, TransactionController
+from arangodb.query.advanced import Query, Traveser
+from arangodb.query.utils.document import create_document_from_result_dict
+from arangodb.query.simple import SimpleQuery
+from arangodb.transaction.controller import Transaction, TransactionController
 
 
 client = Client(hostname='localhost')
@@ -15,6 +16,8 @@ class ExtendedTestCase(unittest.TestCase):
         """
         """
 
+        self.assertEqual(doc1.id, doc2.id)
+
         for prop in doc1.data:
             doc1_val = doc1.data[prop]
             doc2_val = doc2.data[prop]
@@ -33,17 +36,11 @@ class DatabaseTestCase(unittest.TestCase):
 
         database_name = 'test_foo_123'
 
-        try:
-            db = Database.create(name=database_name)
-        except Exception as err:
-            self.fail('Create threw execption: %s' % err.message)
+        db = Database.create(name=database_name)
 
         self.assertIsNotNone(db)
 
-        try:
-            Database.remove(name=database_name)
-        except Exception as err:
-            self.fail('Remove threw execption: %s' % err.message)
+        Database.remove(name=database_name)
 
     def test_get_all_databases(self):
         databases = Database.get_all()
@@ -66,26 +63,17 @@ class CollectionTestCase(unittest.TestCase):
 
         collection_name = 'test_foo_123'
 
-        try:
-            col = Collection.create(name=collection_name)
-        except Exception as err:
-            self.fail('Create threw exception: %s' % err.message)
+        col = Collection.create(name=collection_name)
 
         self.assertIsNotNone(col)
 
-        try:
-            Collection.remove(name=collection_name)
-        except Exception as err:
-            self.fail('Remove threw exception: %s' % err.message)
+        Collection.remove(name=collection_name)
 
     def test_get_collection(self):
 
         collection_name = 'test_foo_123'
 
-        try:
-            col = Collection.create(name=collection_name)
-        except Exception as err:
-            self.fail('Create threw exception: %s' % err.message)
+        col = Collection.create(name=collection_name)
 
         self.assertIsNotNone(col)
 
@@ -95,10 +83,7 @@ class CollectionTestCase(unittest.TestCase):
         self.assertEqual(col.name, retrieved_col.name)
         self.assertEqual(col.type, retrieved_col.type)
 
-        try:
-            Collection.remove(name=collection_name)
-        except Exception as err:
-            self.fail('Remove threw exception: %s' % err.message)
+        Collection.remove(name=collection_name)
 
 
 class DocumentTestCase(unittest.TestCase):
@@ -137,13 +122,33 @@ class AqlQueryTestCase(ExtendedTestCase):
         self.test_2_col = self.db.create_collection('foo_2')
 
         self.col1_doc1 = self.test_1_col.create_document()
-        self.col1_doc1.set(key='ta', value='fa')
+        self.col1_doc1.little_number = 33
+        self.col1_doc1.loved = False
+        self.col1_doc1.small_text = "lll aa"
         self.col1_doc1.save()
 
+        self.col1_doc2 = self.test_1_col.create_document()
+        self.col1_doc2.little_number = 1
+        self.col1_doc2.loved = False
+        self.col1_doc2.small_text = "aaa aa"
+        self.col1_doc2.save()
+
+        self.col1_doc3 = self.test_1_col.create_document()
+        self.col1_doc3.little_number = 3
+        self.col1_doc3.loved = True
+        self.col1_doc3.small_text = "xxx tt"
+        self.col1_doc3.save()
+
         self.col2_doc1 = self.test_2_col.create_document()
-        self.col2_doc1.set(key='ta', value='fa2')
+        self.col2_doc1.little_number = 33
+        self.col2_doc1.loved = False
         self.col2_doc1.save()
 
+        self.col2_doc2 = self.test_2_col.create_document()
+        self.col2_doc2.little_number = 11
+        self.col2_doc2.loved = True
+        self.col2_doc2.save()
+
     def tearDown(self):
         # They need to be deleted
         Collection.remove(name=self.test_1_col.name)
@@ -152,14 +157,83 @@ class AqlQueryTestCase(ExtendedTestCase):
         Database.remove(name=self.database_name)
 
     def test_get_all_doc_from_1_collection(self):
+        q = Query()
+        q.append_collection(self.test_2_col.name)
+        docs = q.execute()
+
+        self.assertEqual(len(docs), 2)
+
+    def test_filter_number_field_in_document(self):
         q = Query()
         q.append_collection(self.test_1_col.name)
+        q.filter(little_number=self.col1_doc3.little_number)
+
+        docs = q.execute()
+
+        self.assertEqual(len(docs), 1)
+
+        doc = docs[0]
+        self.assertDocumentsEqual(doc, self.col1_doc3)
+
+    def test_filter_string_field_in_document(self):
+        q = Query()
+        q.append_collection(self.test_1_col.name)
+        q.filter(small_text=self.col1_doc2.small_text)
+
+        docs = q.execute()
+
+        self.assertEqual(len(docs), 1)
+
+        doc = docs[0]
+        self.assertDocumentsEqual(doc, self.col1_doc2)
+
+    def test_filter_from_multiple_collections(self):
+        q = Query()
+        q.append_collection(self.test_1_col.name)
+        q.append_collection(self.test_2_col.name)
+
+        dynamic_filter_dict = {}
+        col_1_filter_name = "%s__%s" % (self.test_1_col.name, "little_number")
+        col_2_filter_name = "%s__%s" % (self.test_2_col.name, "little_number")
+
+        dynamic_filter_dict[col_1_filter_name] = 33
+        dynamic_filter_dict[col_2_filter_name] = 33
+        q.filter(bit_operator=Query.OR_BIT_OPERATOR, **dynamic_filter_dict)
+
+        docs = q.execute()
+
+        for doc in docs:
+            print(doc.get_attributes())
+
+    def test_exclude_document_from_list(self):
+        q = Query()
+        q.append_collection(self.test_1_col.name)
+        q.exclude(loved=False)
+
         docs = q.execute()
 
         self.assertEqual(len(docs), 1)
 
         doc1 = docs[0]
-        self.assertDocumentsEqual(doc1, self.col1_doc1)
+
+        self.assertDocumentsEqual(doc1, self.col1_doc3)
+
+    def test_sorting_asc_document_list(self):
+        q = Query()
+        q.append_collection(self.test_1_col.name)
+        q.order_by('little_number')
+
+        docs = q.execute()
+
+        self.assertEqual(len(docs), 3)
+
+        doc1 = docs[0]
+        doc2 = docs[1]
+        doc3 = docs[2]
+
+        self.assertDocumentsEqual(doc1, self.col1_doc2)
+        self.assertDocumentsEqual(doc2, self.col1_doc3)
+        self.assertDocumentsEqual(doc3, self.col1_doc1)
 
 
 class SimpleQueryTestCase(ExtendedTestCase):
@@ -209,6 +283,49 @@ class SimpleQueryTestCase(ExtendedTestCase):
         self.assertNotEqual(doc1, doc2)
 
 
+class TraveserTestCase(ExtendedTestCase):
+    def setUp(self):
+        self.database_name = 'testcase_simple_query_123'
+        self.db = Database.create(name=self.database_name)
+
+        # Create collections
+        self.test_1_doc_col = self.db.create_collection('doc_col_1')
+        self.test_1_edge_col = self.db.create_collection('edge_col_1', type=3)
+
+        # Create test data
+        self.doc1 = self.test_1_doc_col.create_document()
+        self.doc1.ta='fa'
+        self.doc1.save()
+
+        self.doc2 = self.test_1_doc_col.create_document()
+        self.doc2.ta='foo'
+        self.doc2.save()
+
+        # Create test relation
+        self.edge1 = self.test_1_edge_col.create_edge(from_doc=self.doc1, to_doc=self.doc2, edge_data={
+            'data': 'in_between'
+        })
+
+    def tearDown(self):
+        # They need to be deleted
+        Collection.remove(name=self.test_1_doc_col.name)
+        Collection.remove(name=self.test_1_edge_col.name)
+
+        Database.remove(name=self.database_name)
+
+    def test_traverse_relation(self):
+        result_list = Traveser.follow(
+            start_vertex=self.doc1.id,
+            edge_collection=self.test_1_edge_col.name,
+            direction='outbound'
+        )
+
+        self.assertEqual(len(result_list), 1)
+
+        result_doc = result_list[0]
+        self.assertDocumentsEqual(result_doc, self.doc2)
+
+
 class CollectionModelTestCase(unittest.TestCase):
     def setUp(self):
         self.database_name = 'testcase_collection_model_123'
@@ -375,19 +492,70 @@ class CollectionModelForeignKeyFieldTestCase(unittest.TestCase):
         TestModel.destroy()
 
 
-class TransactionTestCase(unittest.TestCase):
+class TransactionTestCase(ExtendedTestCase):
     def setUp(self):
-        pass
+
+        self.database_name = 'testcase_transaction_123'
+        self.db = Database.create(name=self.database_name)
+
+        self.operating_collection = 'foo_test'
+        self.test_1_col = Collection.create(name=self.operating_collection)
 
     def tearDown(self):
-        pass
+        Collection.remove(name=self.operating_collection)
+        Database.remove(name=self.database_name)
+
+    def test_create_document(self):
+
+        trans = Transaction(collections={
+            'write': [
+                self.operating_collection,
+            ]
+        })
+
+        # Uses already chosen database as usual
+        collection = trans.collection(name=self.operating_collection)
+        collection.create_document(data={
+            'test': 'foo'
+        })
 
-    def test_create_collection(self):
-        trans = Transaction(collections=[])
-        
         ctrl = TransactionController()
 
+        transaction_result = ctrl.start(transaction=trans)
+
+        transaction_doc = create_document_from_result_dict(transaction_result['result'], self.test_1_col.api)
+
+        created_doc = SimpleQuery.get_by_example(self.test_1_col, example_data={
+            '_id': transaction_doc.id
+        })
+
+        self.assertDocumentsEqual(transaction_doc, created_doc)
+
+    def test_update_document(self):
+
+        doc = self.test_1_col.create_document()
+        doc.foo = 'bar'
+        doc.save()
+
+        trans = Transaction(collections={
+            'write': [
+                self.operating_collection,
+            ]
+        })
+
+        new_foo_value = 'extra_bar'
+
+        collection = trans.collection(self.operating_collection)
+        collection.update_document(doc_id=doc.id, data={
+            'foo': new_foo_value
+        })
+
+        ctrl = TransactionController()
         ctrl.start(transaction=trans)
 
+        doc.retrieve()
+
+        self.assertEqual(doc.foo, new_foo_value)
+
 if __name__ == '__main__':
     unittest.main()
\ No newline at end of file
